package com.ekexiu.console.system.service;

import com.ekexiu.console.system.dao.QuestionDao;
import com.ekexiu.console.system.pojo.AnswerInfo;
import com.ekexiu.console.system.pojo.QuestionInfo;
import com.ekexiu.console.util.HttpService;
import com.ekexiu.console.util.HttpServiceImpl;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.JpgUtil;
import org.jfw.util.ListUtil;
import org.jfw.util.PageQueryResult;
import org.jfw.util.io.IoUtil;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

/**
 * Created by TT on 2018/1/26.
 */
@Path("/qa")
public class QuestionService {
    private static HttpService http = new HttpServiceImpl();
    private File imgPath;
    private int imgMaxWidth = 70;

    @Autowrie
    private QuestionDao questionDao;

    public QuestionDao getQuestionDao() {
        return questionDao;
    }

    public void setQuestionDao(QuestionDao questionDao) {
        this.questionDao = questionDao;
    }

    public File getImgPath() {
        return imgPath;
    }

    public void setImgPath(File imgPath) {
        this.imgPath = imgPath;
    }

    public int getImgMaxWidth() {
        return imgMaxWidth;
    }

    public void setImgMaxWidth(int imgMaxWidth) {
        this.imgMaxWidth = imgMaxWidth;
    }

    @Path("/pqAnswer")
    @Get
    public PageQueryResult<AnswerInfo> pqAnswer(@JdbcConn Connection con, @Nullable String questionTitle, @Nullable String respondent, @Nullable String cnt, int orderBy, @DefaultValue("1") int pageNo, @DefaultValue("10") int pageSize) throws SQLException {
        if (orderBy == 1) {
            return this.questionDao.answerByTime(con, questionTitle == null ? null : "%" + questionTitle + "%", respondent == null ? null: "%" + respondent + "%", cnt == null ? null : "%" + cnt + "%", pageSize, pageNo);
        } else if (orderBy == 2) {
            return this.questionDao.answerByAgree(con, questionTitle == null ? null : "%" + questionTitle + "%", respondent == null ? null : "%" + respondent + "%", cnt == null ? null : "%" + cnt + "%", pageSize, pageNo);
        } else if (orderBy == 3) {
            return this.questionDao.answerByBallot(con, questionTitle == null ? null : "%" + questionTitle + "%", respondent == null ? null : "%" + respondent + "%", cnt == null ? null : "%" + cnt + "%", pageSize, pageNo);
        }
        return null;
    }

    @Path("/answer")
    @Get
    public AnswerInfo answer(@JdbcConn Connection con, String id) throws SQLException {
        return this.questionDao.queryAnswer(con, id);
    }

    @Post
    @Path("/answer/modify")
    public void answerUpdate(@JdbcConn(true) Connection con, String id, String cnt) throws SQLException {
        this.questionDao.updateAnswer(con, id, cnt);
    }

    @Path("/deleteAnswer")
    @Get
    public int deleteAnswer(@JdbcConn(true) Connection con, String id,String qid) throws SQLException {
        if (questionDao.deleteAnswer(con, id, qid) > 0) {
            questionDao.decQuestionReply(con, qid);
            return 1;
        }
        return 0;
    }

    @Path("/pqQuestion")
    @Get
    public PageQueryResult<QuestionInfo> pqQuestion(@JdbcConn Connection con, @Nullable String title, @Nullable String requester, int orderBy, @DefaultValue("1") int pageNo, @DefaultValue("10") int pageSize) throws SQLException {
        if (orderBy == 1) {
            return this.questionDao.questionByTime(con, title == null ? null : "%" + title + "%", requester == null ? null : "%" + requester + "%", pageSize, pageNo);
        } else if (orderBy == 2) {
            return this.questionDao.questionByPV(con, title == null ? null : "%" + title + "%", requester == null ? null : "%" + requester + "%", pageSize, pageNo);
        } else if (orderBy == 3) {
            return this.questionDao.questionByReply(con, title == null ? null : "%" + title + "%", requester == null ? null : "%" + requester + "%", pageSize, pageNo);
        }
        return null;
    }

    @Path("/question")
    @Get
    public QuestionInfo question(@JdbcConn Connection con, String id) throws SQLException {
        return this.questionDao.queryQuestion(con, id);
    }

    @Path("/question/modify")
    @Post
    public void questionUpdata(@JdbcConn(true) Connection con, String id, String cnt,String img) throws SQLException,IOException {
        this.saveSmallImg(img);
        this.questionDao.updateQuestion(con, id, cnt,img);
    }

    @Path("/question/keys")
    @Post
    public void questionKey(@JdbcConn(true)Connection con,String id,String keys)throws SQLException {
        this.questionDao.updataQetKey(con, id, keys);
    }

    @Path("/deleteQuestion")
    @Get
    public void deleteQuestion(@JdbcConn(true) Connection con, String id) throws SQLException {
        this.questionDao.logicDeleteQuestion(con, id);
    }



    private void saveSmallImg(String imgs) throws IOException {
        if (imgs != null) {
            List<String> list = ListUtil.splitTrimExcludeEmpty(imgs, ',');
            if (list.size() > 0) {
                String oname = list.get(0);
                if (oname != null) {
                    if (oname.startsWith("/")) {
                        oname = oname.substring(1);
                        int idx = oname.indexOf('.');
                        if (idx > 0) {
                            String dname = oname.substring(0, idx) + "_s.jpg";
                            File dest = new File(this.imgPath, dname);
                            if (dest.exists())
                                return;
                            File src = new File(this.imgPath, oname);
                            if (src.exists()) {
                                byte[] obs = IoUtil.readStream(new FileInputStream(src), true);
                                obs = JpgUtil.read(obs);
                                FileOutputStream out = new FileOutputStream(dest);
                                try {
                                    JpgUtil.scalingZoom(new ByteArrayInputStream(obs), out, this.imgMaxWidth);
                                    out.flush();
                                } finally {
                                    IoUtil.close(out);
                                }
                            }
                        }
                    }
                }
            }
        }
    }


}
